
log=/var/log/messages
bios_util_dir=$(dirname ${0})
bios_util_dir=$(realpath ${bios_util_dir})

log_output()
{
	date_time=$(date | awk '{print $2" " $3 " " $4" "}')
	echo -e ${date_time}"\t"$1 | tee -a $log 
}

usage()
{
	echo "Usage: `basename $0` <full path to image> [me | params] [no_reboot]"
	echo -e "`basename $0` <full path to BIOS image>\t - BIOS update"
	echo -e "`basename $0` <full path to SPI image> me\t - Full SPI flash update: BIOS + me"
	echo -e "`basename $0` <full path to BIOS image> params\t - BIOS paramters only update"
	echo -e "no_reboot\t\t\t - dont't make automatic reboot/power cycle at the end." 	 
}

make_power_cycle()
{
	lpc_base=0x2500
	me_reboot_offs=0x32
	me_reboot_wp_offs=0x33
	me_reboot_bit=6
	iorw_util=iorw

	lpc_mod=$(lsmod | grep lpci2c)
	len=${#lpc_mod}
	if [ $len -lt 6 ]; then
		log_output "lpci2c module is not loaded. Can not make power cycle"
		return 1
	fi

	let "me_reboot_wp_reg=$lpc_base+$me_reboot_wp_offs"
	val=$($iorw_util -r -b $me_reboot_wp_reg -l 1 | awk '{print $5}')
	let "mask=~(1<<$me_reboot_bit)"
	let "val&=$mask"
	$iorw_util -w -b $me_reboot_wp_reg -l 1 -v $val

	let "me_reboot_reg=$lpc_base+$me_reboot_offs"
	val=$($iorw_util -r -b $me_reboot_reg -l 1 | awk '{print $5}')
	let "mask=(1<<$me_reboot_bit)"
	let "val|=$mask"
	let "me_reboot_reg=$lpc_base+$me_reboot_offs"
	$iorw_util -w -b $me_reboot_reg -l 1 -v $val
}

if [ $# -ge 1 ]; then
	power_cycle=0
	no_reboot=0
	bios_img=`pwd`/${1}
	bios_dir=$(dirname ${1})
	bios_name=$(basename ${1})
	md5_sum=$bios_name".md5"

# Check input parameters
	if [ $# -ge 2 ]; then
		if [ $2 == "me" ]; then
			util_params=" /p /b /n /x /me"
			power_cycle=1
		elif [ $2 == "params" ]; then
			util_params=" /n"
		elif [ $2 == "no_reboot" ]; then
			util_params=" /p /b /n /x"
			no_reboot=1
		else
			log_output "Error: ${0} bad paramters"
			usage
			exit 1
		fi
	else
		util_params=" /p /b /n /x"
	fi
	if [ $# -eq 3 ] && [ $3 == "no_reboot" ]; then
		no_reboot=1
	fi
	

# Check md5	digest
	cd $bios_dir	
	echo ""
	if [ ! -f $md5_sum ]; then
		log_output "Error: ${md5_sum} wasn't found"
		exit 1
	fi

	md5sum -c $md5_sum
	rc=$?
	if [ $rc -ne 0 ]; then
		log_output "Error: bios image ${bios_img} is corrupted, md5 is failed"
		exit 1
	fi

	cd $bios_util_dir
	if [ ! -f $bios_img ]; then
		log_output "Error: ${bios_img} wasn't found"
		exit 1
	fi

	bios_file=${bios_img}
	/usr/bin/afulnx_26_64 ../../$bios_file $util_params
	rc=$?
	if [ $rc -ne 0 ]; then
		log_output "BIOS ${bios_img} update failed"
                exit 1
	else
		log_output "BIOS ${bios_img} update success"
		if [ $no_reboot -eq 0 ]; then
			if [ $power_cycle -eq 1 ]; then
				log_output "Power cycle will be done after 5 sec"
				sleep 5
				make_power_cycle
			else
				log_output "Reboot will be done after 5 sec"
				sleep 5
				reboot
			fi
		fi
                exit 0
	fi
else
	log_output "Error: Full path to bios image should be provided"
	usage
	exit 1
fi
