/*
 *
 * Copyright (C) Mellanox Technologies Ltd. 2001-2014.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __MLNX_CPLD_H__
#define __MLNX_CPLD_H__

#define CPLD_JTAG_DEVICE    "/dev/cpld_jtag"
#define CPLD_JTAG_DEVICE_MAJOR  240

#define DASD_IOCTL_LETTER 'D'
#define IOCTL_CPLD_JTAG_SET     _IOWR(DASD_IOCTL_LETTER, 0, int)
#define IOCTL_CPLD_GPIO_READ    _IOWR(DASD_IOCTL_LETTER, 1, int)
#define IOCTL_CPLD_GPIO_WRITE   _IOWR(DASD_IOCTL_LETTER, 2, int)
#define IOCTL_CPLD_JTAG_IO      _IOWR(DASD_IOCTL_LETTER, 3, int)
#define IOCTL_CPLD_SELECT       _IOWR(DASD_IOCTL_LETTER, 4, int)

#define CPLD_JTAG_DIS   0
#define CPLD_JTAG_ENB   1

#define CPLD_JTAG_IO_ERR    -1
#define CPLD_JTAG_IO_OK     0
#define CPLD_JTAG_IO_WARN   1

enum gpio_state { GPIO_ORIG_STATE, GPIO_JTAG_STATE };

struct jtag_set_ioctl_param {
    unsigned char status;
};

struct read_ioctl_param {
	unsigned char pin;
	unsigned char dir;
	unsigned char data;
};

struct write_ioctl_param {
	unsigned char pin;
	unsigned char data;
};

struct jtag_io_ioctl_param {
    struct write_ioctl_param tms;
    struct write_ioctl_param tdi;
    struct write_ioctl_param tck;
    char rc;
};

struct cpld_select_ioctl_param {    
    unsigned char cpld;
    unsigned char status;
};

#ifdef __KERNEL__

#define CPLD_DEVICE_NAME "cpld_jtag"

#define CPLD_IOEXP_CFG_REG  0x1B    /* I2C IO expansion config register */
#define CPLD_IOEXP_OUT_REG  0x0B    /* I2C IO expansion output port register */
#define CPLD_IOEXP_JTAG_BIT    0    /* IO expansion JTAG enable bit/pin */
#define CPLD_JTAG_REG1       0x8    /* JTAG CPLD select */
#define CPLD_JTAG_MODE1      0x30
#define CPLD_JTAG_MASK1      0xC8

#define CPLD_JTAG_REG2      0x34    /* JTAG CPLD select */
#define CPLD_JTAG_MODE2     0x1
#define CPLD_JTAG_MASK2     0x1F

/* Relevant only for PPC systems */
#define CPLD_JTAG_ENB_MODE1 0   /* This is mapping corresponds to old Jackal CPLDs */
#define CPLD_JTAG_ENB_MODE2 1   /* This is mapping corresponds to new Coyote / Coyote-X CPLDs */

extern int dbg_lvl;
#define CPLD_DBG_MAX_LVL        4
#define CPLD_LOG_ERROR(format, args...) printk(KERN_ERR "CPLD: " format, ## args)
#ifdef  CPLD_DEBUG
#define CPLD_LOG_DBG(level, format, args...)  if (dbg_lvl > 0 &&  dbg_lvl >= level) {printk("CPLD DBG: " format, ## args);}
#else
#define CPLD_LOG_DBG(level, format, args...)
#endif

/* Default parameters, currently for IBM blade */
#define DFLT_CPLD_I2C_BUS   5
#define DFLT_CPLD_I2C_ADDR  0x60
#define DFLT_CPLD_IO_PORT   0x820   /* Virtual i2c bus & address, B0 - i2c address, B1-2 - bus id */
#define CPLD_I2C_BUS_MASK   0xFFFF00
#define CPLD_I2C_ADDR_MASK  0x7F

#define DFLT_CPLD_DBG_LVL   0    /* Disable all - 0 */

#ifdef CONFIG_X86
#define CPLD_GPIO_MAX       6
#else
#define CPLD_GPIO_MAX       4
#endif

#define CPLD1               1
#define CPLD2               2
#define CPLD3               3

#define CPLD_ID_REG         0x20
#define CPLD_VER_REG        0x21
#define CPLD_MAX_REG        0x37

#define CPLD_ID_IBM_BLADE   0x22

#define CPLD_RC_FAILURE     -1
#define CPLD_RC_OK          0

struct cpld_stat {
    u32 ioctl_jtag_set_num;
    u32 ioctl_jtag_io_num;
    u32 ioctl_read_num;
    u32 ioctl_cpld_select_num;
#ifdef CPLD_WARNING
    u32 warn_num;
#endif
};

struct cpld_device_t {
    struct platform_device* plat_dev;
    spinlock_t cpld_lock;
    enum gpio_state gpio_st;
    struct cpld_stat stat;
    struct attribute_group attr_grp;
    struct kobject* reg_dir;
    struct kobject* reg[CPLD_MAX_REG];
#ifndef CONFIG_X86
    u8 i2c_addr;                    /* cpld i2c address */
    u8 i2c_bus;                     /* cpld i2c bus */
    struct i2c_adapter *adapt;      /* cpld i2c adapter */
    u8 io_i2c_addr;                 /* i2c address of IO port expansion */
    u8 io_i2c_bus;                  /* i2c bus of IO port expansion */
    struct i2c_adapter *io_adapt;   /* i2c adapter o IO port expansion */
#endif
};
 
extern struct cpld_device_t* cpld_device;

#endif /* __KERNEL__ */

#endif /* __MLNX_CPLD_H__ */
