/**
 *
 * Copyright (C) 2013-2014, Mellanox Technologies Ltd.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __CPLD_GPIO_H__
#define __CPLD_GPIO_H__

#define GPIO_GROUP_MAX	2

#define GPIO0_BASE	0x4EF600B00ULL  /* 36-bit address */ 
#define GPIO1_BASE	0x4EF600C00ULL  /* 36-bit address */
#define GPIO_SIZE	128

#define GPIOx_OR	0x00		/* GPIO Output Register */
#define GPIOx_TCR	0x04		/* GPIO Three-State Control Register */
#define GPIOx_OSL	0x08		/* GPIO Output Select Register (Bits 0-31) */
#define GPIOx_OSH	0x0C		/* GPIO Ouput Select Register (Bits 32-63) */
#define GPIOx_TSL	0x10		/* GPIO Three-State Select Register (Bits 0-31) */
#define GPIOx_TSH	0x14		/* GPIO Three-State Select Register  (Bits 32-63) */
#define GPIOx_ODR	0x18		/* GPIO Open drain Register */
#define GPIOx_IR	0x1C		/* GPIO Input Register */
#define GPIOx_RR1	0x20		/* GPIO Receive Register 1 */
#define GPIOx_RR2	0x24		/* GPIO Receive Register 2 */
#define GPIOx_RR3	0x28		/* GPIO Receive Register 3 */
#define GPIOx_IS1L	0x30		/* GPIO Input Select Register 1 (Bits 0-31) */
#define GPIOx_IS1H	0x34		/* GPIO Input Select Register 1 (Bits 32-63) */
#define GPIOx_IS2L	0x38		/* GPIO Input Select Register 2 (Bits 0-31) */
#define GPIOx_IS2H	0x3C		/* GPIO Input Select Register 2 (Bits 32-63) */
#define GPIOx_IS3L	0x40		/* GPIO Input Select Register 3 (Bits 0-31) */
#define GPIOx_IS3H	0x44		/* GPIO Input Select Register 3 (Bits 32-63) */

#define GPIO_OR(x)	((x)+GPIOx_OR)		/* GPIO Output Register */
#define GPIO_TCR(x)	((x)+GPIOx_TCR)		/* GPIO Three-State Control Register */
#define GPIO_OS(x)	((x)+GPIOx_OSL)		/* GPIO Output Select Register High or Low */
#define GPIO_IR(x)	((x)+GPIOx_IR)		/* GPIO Input Register  */
#define GPIO_TS(x)	((x)+GPIOx_TSL)		/* GPIO Three-state Control Reg High or Low */
#define GPIO_IS1(x)	((x)+GPIOx_IS1L)	/* GPIO Input register1 High or Low */
#define GPIO_IS2(x)	((x)+GPIOx_IS2L)	/* GPIO Input register2 High or Low */
#define GPIO_IS3(x)	((x)+GPIOx_IS3L)	/* GPIO Input register3 High or Low */

#define GPIO_MAX	32
#define GPIO_ALT1_SEL	0x40000000
#define GPIO_ALT2_SEL	0x80000000
#define GPIO_ALT3_SEL	0xc0000000
#define GPIO_IN_SEL	0x40000000
#define GPIO_MASK	0xc0000000

#define GPIO_VAL(gpio)	(0x80000000 >> (gpio))

enum gpio_select { GPIO_SEL, GPIO_ALT1, GPIO_ALT2, GPIO_ALT3 };
enum gpio_driver { GPIO_DIS, GPIO_IN, GPIO_OUT, GPIO_BI };
enum gpio_out	 { GPIO_OUT_0, GPIO_OUT_1, GPIO_OUT_NO_CHG };

struct gpio_param_type {
    u8              pin;
	enum gpio_driver    in_out;	/* Driver Setting		*/
	enum gpio_select    alt_nb;	/* Selected Alternate   */
	enum gpio_out       out_val;/* Default Output Value	*/
};

int cpld_gpio_init(void);
void cpld_gpio_release(void);
void cpld_gpio_config(struct gpio_param_type* cpld_gpio);
void gpio_write_bit(u8 pin, u8 val);
int gpio_read_bit(u8 pin);    /* cpld_gpio_read_out_bit */
int gpio_read_in_bit(u8 pin);
void gpio_config_pin(struct gpio_param_type* cpld_gpio);

/* There are 2 different sets of pins that is used for JTAG simulation.
   Set 0 (defult):  TDO - pin 28 & TDI - pin 29. It's used in IBM blade.
   Set 1:           TDI - pin 28 & TDO - pin 29. It's used on Jackal. */
#define GPIO27  27      // TMS
#define GPIO28  28      // TDO or TDI
#define GPIO29  29      // TDI or TDO
#define GPIO30  30      // TCK

#define CPLD_GPIO_TBL_MAX   2

#define CONFIG_CPLD_ORIG_GPIO_TABLE {   \
{GPIO27, GPIO_IN , GPIO_ALT1, GPIO_OUT_0}, /* GPIO27 IRQ(0)				*/	\
{GPIO28, GPIO_IN , GPIO_ALT1, GPIO_OUT_0}, /* GPIO28 IRQ(1)				*/	\
{GPIO29, GPIO_IN , GPIO_ALT1, GPIO_OUT_0}, /* GPIO29 IRQ(2)				*/	\
{GPIO30, GPIO_IN,  GPIO_SEL , GPIO_OUT_1}, /* GPIO30 PerPar0	DMAReq2		IRQ(7)*/ \
}

#define CONFIG_CPLD_JTAG_GPIO_TABLE0 {   \
{GPIO27, GPIO_OUT, GPIO_SEL , GPIO_OUT_1},   /* TMS */   \
{GPIO28, GPIO_IN,  GPIO_SEL , GPIO_OUT_0},   /* TDO */   \
{GPIO29, GPIO_OUT, GPIO_SEL , GPIO_OUT_1},   /* TDI */   \
{GPIO30, GPIO_OUT, GPIO_SEL , GPIO_OUT_1},   /* TCK */   \
}

#define CONFIG_CPLD_JTAG_GPIO_TABLE1 {   \
{GPIO27, GPIO_OUT, GPIO_SEL , GPIO_OUT_1}, /* GPIO27 CPU_CPLD_TMS   */	\
{GPIO28, GPIO_OUT, GPIO_SEL , GPIO_OUT_1}, /* GPIO28 CPU_CPLD_TDI   */	\
{GPIO29, GPIO_IN,  GPIO_SEL , GPIO_OUT_0}, /* GPIO29 CPU_CPLD_TDO   */	\
{GPIO30, GPIO_OUT, GPIO_SEL , GPIO_OUT_1}, /* GPIO30 CPU_CPLD_TCK   */	\
}

#endif /* __CPLD_GPIO_H__ */
