/*
 *
 * Copyright (C) Mellanox Technologies Ltd. 2001-2014.  ALL RIGHTS RESERVED.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

#ifndef __GPIO_PCH_H__
#define __GPIO_PCH_H__

#define GPIO_RC_FAILURE     -1
#define GPIO_RC_OK          0

#define GPIO_DEVICE_NAME    "gpio_pch"

#define GPIO_MAX_NUM    76
#define GPIO_IOLEN      128

#define LPC_IFC_BUS_ID      0
#define LPC_IFC_SLOT_ID     31
#define LPC_IFC_FUNC_ID     0

#define GPIOBASE_OFFS       0x48
#define GPIOBASE_MASK       0xFF80
#define GPIOBASE_SHIFT      7

#define GPIO_USE_SEL1_OFFS  0x0
#define GPIO_USE_SEL2_OFFS  0x30
#define GPIO_USE_SEL3_OFFS  0x40
#define GPIO_IO_SEL1_OFFS   0x4
#define GPIO_IO_SEL2_OFFS   0x34
#define GPIO_IO_SEL3_OFFS   0x44
#define GPIO_LVL1_OFFS      0xC
#define GPIO_LVL2_OFFS      0x38
#define GPIO_LVL3_OFFS      0x48
#define GPIO_RST1_OFFS      0x60
#define GPIO_RST2_OFFS      0x64
#define GPIO_RST3_OFFS      0x68

#define GPIO_VAL(gpio, val)	((val & (1 << gpio))>>gpio)

#define GPIO_DBG_MAX_LVL        4
#define GPIO_DFLT_DBG_LVL       0
#define GPIO_LOG_ERROR(format, args...) printk(KERN_ERR "GPIO: " format, ## args)
#ifdef  GPIO_DEBUG
#define GPIO_LOG_DBG(level, format, args...)  if (dbg_lvl > 0 &&  dbg_lvl >= level) {printk("GPIO DBG: " format, ## args);}
#else
#define GPIO_LOG_DBG(level, format, args...)
#endif

struct gpio_device_t {
    struct platform_device* plat_dev;
    struct resource	* resource;
    spinlock_t lock;
    u32 gpio_base;
    struct attribute_group gpio_attr_grp;
    struct kobject* gpio_dir;
    struct kobject* gpio_sysfs[(GPIO_MAX_NUM-1)];
};

extern struct gpio_device_t* gpio_device;
extern int dbg_lvl;

void gpio_data_init(struct gpio_device_t* gpio_dev, struct pci_dev* pdev);
inline u32 gpio_read_reg(u8 gpio, enum gpio_reg_type type);
inline void gpio_write_reg(u8 gpio, enum gpio_reg_type type, u32 val);

#endif
