#ifndef __MLNX_I2C_IF_H_
#define __MLNX_I2C_IF_H_

//#define SX_CACHE_OPTIMIZED_2 1
#define SX_CACHE_OPTIMIZED 1

#define SX_ROUTE_DEPTH 4

#define GO_BIT_STAT_OK    0
#define GO_BIT_STAT_STUCK 1
#define PROBE_STAT_OK     0
#define PROBE_STAT_FAILED 1
#define HCR_ACCESS_OK     0
#define HCR_ACCESS_NOT_OK 1
#define LAST_I2C_ACCESS_FAILED (-1)

struct i2c_mux_priv {
	struct i2c_adapter adap;
	struct i2c_algorithm algo;
	struct i2c_adapter *parent;
	void *mux_dev;	/* the mux chip/device */
	u32  chan_id;	/* the channel id */
	int (*select)(struct i2c_adapter *, void *mux_dev, u32 chan_id);
	int (*deselect)(struct i2c_adapter *, void *mux_dev, u32 chan_id);
};

struct i2c_mux_priv_cache {
	struct i2c_mux_priv *priv;
	int (*saved_deselect)(struct i2c_adapter *, void *mux_dev, u32 chan_id);
};

extern int __last_accessed_bus_id;
extern struct i2c_mux_priv_cache *__p_last_bus_cache_path[SX_ROUTE_DEPTH - 1];

static inline int mlnx_i2c_break_nailed_up_route(struct i2c_adapter *adap)
{
	int count;

	if(!__p_last_bus_cache_path[SX_ROUTE_DEPTH - 2])
		return 0;

	if (__last_accessed_bus_id == adap->nr) /* Stay selected */
		return 0;

	if(!__p_last_bus_cache_path[SX_ROUTE_DEPTH - 2])
		return 0;

	/* Break previouse route */
	for (count = SX_ROUTE_DEPTH - 2; count >= 0; count--) {
		struct i2c_mux_priv *priv = __p_last_bus_cache_path[count]->priv;
		if (priv) {
                	if(__p_last_bus_cache_path[count]->saved_deselect) {
                		__p_last_bus_cache_path[count]->saved_deselect(priv->parent, priv->mux_dev, priv->chan_id);
                		priv->deselect = __p_last_bus_cache_path[count]->saved_deselect;
                	}
			/* Clean saved cache */
			__p_last_bus_cache_path[count]->priv = NULL;
			__p_last_bus_cache_path[count]->saved_deselect = NULL;
		}
	}
	return 1;
}

static inline void mlnx_i2c_save_nailed_up_route(struct i2c_adapter *adap)
{
        struct i2c_adapter *parent = NULL;
	struct i2c_mux_priv *priv = NULL;
	int count = SX_ROUTE_DEPTH - 2;
	int rc = 1;

	if(!adap) {
		__last_accessed_bus_id = LAST_I2C_ACCESS_FAILED;
		return;
	}
	if (__last_accessed_bus_id != LAST_I2C_ACCESS_FAILED) {
		rc = mlnx_i2c_break_nailed_up_route(adap);
		if (rc == 0)
			return;
	}
	__last_accessed_bus_id = adap->nr;
 
	/* Set path for access via virtual i2c */
	while(adap) {
		priv = adap->algo_data;
		if (!priv)
			break;
		parent = priv->parent;
		if (!parent)
			break;
		__p_last_bus_cache_path[count]->priv = priv;
		__p_last_bus_cache_path[count]->saved_deselect = priv->deselect;
		priv->deselect = NULL;
		count--;
		adap = parent;
	}
}

int sx_i2c_get_probe_status(struct i2c_client *client, 
				u8 *probe_status, 
				int *probe_cntr, 
				u8 *go_bit_status,
				u8 *hcr_acess_status);

#endif /* __MLNX_I2C_IF_ */
