#ifndef __SX_I2C_IF_H_
#define __SX_I2C_IF_H_

#if 0 
/* Kernel SwitchX I2C common access API */


/* 
  Description: writing data blok to i2c device
  Input:
               i2c_dev_id - device id of particular switchx
               offset     - offset from which device to be written
               len        - number of bytes to be written
               in_out_buf - point to input write buffer
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note: each transfer is protected by a number of mutex. This number depends on
        the number of i2c muxes on the data transfer way to the targeted device.
*/
int sx_i2c_write(int i2c_dev_id, int offset, int len, u8 *in_out_buf);

/* 
  Description: reading data blok from i2c device
  Input:
               i2c_dev_id - device id of particular switchx
               offset     - offset from which device to be read
               len        - number of bytes to be read
               in_out_buf - pointer to input read buffer
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note: each transfer is protected by a number of mutex. This number depends on
        the number of i2c muxes on the data transfer way to the targeted device.
*/
int sx_i2c_read(int i2c_dev_id, int offset, int len, u8 *in_out_buf);

/* 
  Description: writing double word to i2c device
  Input:
               i2c_dev_id - device id of particular switchx
               offset     - offset from which device to be written
               val        - pointer to input double word to where data to be put
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note: each transfer is protected by a number of mutex. This number depends on
        the number of i2c muxes on the data transfer way to the targeted device.
*/
int sx_i2c_write_dword(int i2c_dev_id, int offset, u32 val);

/* 
  Description: reading double word from i2c device
  Input:
               i2c_dev_id - device id of particular switchx
               offset     - offset from which device to be read
               val        - pointer to input double word from where data to be get
  Return code:
               0 - on success
               negative values (standard Linux error code) on failure.
  Note: each transfer is protected by a number of mutex. This number depends on
        the number of i2c muxes on the data transfer way to the targeted device.
*/
int sx_i2c_read_dword(int i2c_dev_id, int offset, u32 *val);
#endif

struct sx_i2c_ifc{
	int (*write)(int i2c_dev_id, int offset, int len, u8 *in_out_buf);
	int (*read)(int i2c_dev_id, int offset, int len, u8 *in_out_buf);
	int (*write_dword)(int i2c_dev_id, int offset, u32 val);
	int (*read_dword)(int i2c_dev_id, int offset, u32 *val);
	int (*enforce)(int i2c_dev_id);
	int (*release)(int i2c_dev_id);
	int (*get_local_mbox)(int i2c_dev_id, u32 *mb_size_in, u32 *mb_offset_in, u32 *mb_size_out, u32 *mb_offset_out);
	int (*get_fw_rev)(int i2c_dev_id, u64* fw_rev);
	//int (*set_go_bit_stuck)(int i2c_dev_id);
};
/*
void sx_dpt_reg_i2c_ifc(int (*i2c_write)(int i2c_dev_id, int offset, int len, u8 *in_out_buf),
		int (*i2c_read)(int i2c_dev_id, int offset, int len, u8 *in_out_buf),
		int (*i2c_write_dword)(int i2c_dev_id, int offset, u32 val),
		int (*i2c_read_dword)(int i2c_dev_id, int offset, u32 *val) );
*/
void sx_dpt_reg_i2c_ifc(struct sx_i2c_ifc *i2c_ifc);

void sx_dpt_dereg_i2c_ifc(void);

#endif /* __SX_I2C_IF_ */
