/*
*
* Copyright (C) Mellanox Technologies Ltd. 2001-2014.  ALL RIGHTS RESERVED.
*
* This software product is a proprietary product of Mellanox Technologies Ltd.
* (the "Company") and all right, title, and interest in and to the software product,
* including all associated intellectual property rights, are and shall
* remain exclusively with the Company.
*
* This software product is governed by the End User License Agreement
* provided with the software product.
*
*/

#ifndef __CPLD_H__
#define __CPLD_H__

#define ALL_CPLD	"ALL"
#define CPLD_TOR	"CPLD_TOR"
#define CPLD_PORT1	"CPLD_PORT1"
#define CPLD_PORT2	"CPLD_PORT2"
#define CPLD_MEZZ	"CPLD_MEZZ"
#define CPLD_MNGB	"CPLD_MNGB"		// Management Board
#define CPLD_SWB	"CPLD_SWB"		// Switch Board

#define	SYST_JACKAL		"SX1016"
#define SYST_DINGO		"SX1012"
#define SYST_COYOTE		"SX1024"
#define SYST_COYOTEX	"MSX1400"
#define SYST_PIRANHAX	"MSX1700"
#define SYST_NEPTUNE	"MSX6710"

#define CPLD1_VERSION_REG       0x0
#define CPLD2_VERSION_REG       0x1
#define CPLD3_VERSION_REG       0x2
#define CPLD_MEZZ_VERS_REG      0x3
#define CPLD_FU_REG             0x34    /* Field Upgrade */

#define CPLD_MAX_NUM	4
#define DFLT_ADDR_WDTH	1
#define DFLT_ACCESS_LEN	1
#define CPLD_BASE_ADDR	0x2500

enum system_model {
    Incorrect_syst,
	Jackal,
	Dingo,
	Coyote,
	Coyotex,
	Piranhax,
	Neptune
};

enum cpld_type {
    Incorrect_cpld,
	Cpld_tor,
	Cpld_port1,
	Cpld_port2,
	Cpld_mezz,
	Cpld_mngb,
	Cpld_swb,
	All
};

enum access_type {
    no_access,
	i2c_access,
	lpc_access,
	fw_access
};

enum rw_operation {
    write_op,
    read_op
};

struct cpld_info {
    const char* name;       // CPLD name
    enum cpld_type  type;   // CPLD type
	uint16_t   bus;     	// Bus where device is set for I2C device or base address for LPC device
    uint8_t    addr_idx;   // Slave address for I2C device or FW idx for FW access
    enum access_type  acc_type;   // I2C / LPC or FW access
};

struct system_info {
	const char* syst_name;
	enum system_model syst;
	uint8_t cpld_num;
	struct cpld_info cplds[CPLD_MAX_NUM];
};

#define NULL_CPLD   {"", Incorrect_cpld, 0xff, 0xff, no_access }

struct system_info jackal_info = {
	SYST_JACKAL, Jackal, 3,
	{{CPLD_TOR, Cpld_tor, 5, 0x60, i2c_access},
	{CPLD_PORT1, Cpld_port1, 0xff, 0, fw_access},
	{CPLD_PORT2, Cpld_port2, 0xff, 1, fw_access},
	NULL_CPLD}
};

struct system_info dingo_info = {
	SYST_DINGO, Dingo, 1,
	{{CPLD_TOR, Cpld_tor, 5, 0x60, i2c_access},
	NULL_CPLD,
	NULL_CPLD,
	NULL_CPLD}
};

struct system_info coyote_info = {
	SYST_COYOTE, Coyote, 3,
	{{CPLD_TOR, Cpld_tor, 5, 0x60, i2c_access},
	{CPLD_PORT1, Cpld_port1, 0xff, 0, fw_access},
	{CPLD_PORT2, Cpld_port2, 0xff, 1, fw_access},
	NULL_CPLD}
};

struct system_info coyotex_info = {
	SYST_COYOTEX, Coyotex, 4,
	{{CPLD_TOR, Cpld_tor, CPLD_BASE_ADDR, 0xff, lpc_access},
	{CPLD_PORT1, Cpld_port1, 0xff, 0, fw_access},
	{CPLD_PORT2, Cpld_port2, 0xff, 1, fw_access},
	{CPLD_MEZZ, Cpld_mezz, CPLD_BASE_ADDR, 0xff, lpc_access}}
};

struct system_info piranhax_info = {
	SYST_PIRANHAX, Piranhax, 3,
	{{CPLD_TOR, Cpld_tor, CPLD_BASE_ADDR, 0xff, lpc_access},
	{CPLD_PORT1, Cpld_port1, 0xff, 0, fw_access},
	{CPLD_MEZZ, Cpld_mezz, CPLD_BASE_ADDR, 0xff, lpc_access},
	NULL_CPLD}
};

struct system_info neptune_info = {
	SYST_NEPTUNE, Neptune, 3,
	{{CPLD_SWB, Cpld_swb, CPLD_BASE_ADDR, 0xff, lpc_access},
	{CPLD_PORT1, Cpld_port1, 0xff, 0, fw_access},
	{CPLD_MNGB, Cpld_mngb, CPLD_BASE_ADDR, 0xff, lpc_access},
	NULL_CPLD}
};

#endif
